@@ -36,7 +36,8 @@ fn main() -> wry::Result<()> {
             </div>
           </div>
           <div>
-            WRYYYYYYYYYYYYYYYYYYYYYY!
+            <p> WRYYYYYYYYYYYYYYYYYYYYYY! </p>
+            <button style="cursor: pointer"> Hover me! </button>
           </div>
         </body>
       "#;
@@ -144,7 +145,7 @@ fn main() -> wry::Result<()> {
             *control_flow = ControlFlow::Exit
           }
         }
-        WindowEvent::Resized(_) | WindowEvent::Moved(_) => {
+        WindowEvent::Resized(_) => {
           let _ = webviews[&window_id].resize();
         }
         _ => (),
