@@ -14,17 +14,15 @@ fn main() -> wry::Result<()> {
   #[cfg(target_os = "linux")]
   use wry::application::platform::unix::WindowBuilderExtUnix;
   #[cfg(target_os = "windows")]
-  use wry::application::platform::windows::SystemTrayExtWindows;
-  #[cfg(target_os = "windows")]
   use wry::application::platform::windows::WindowBuilderExtWindows;
   use wry::{
     application::{
       accelerator::{Accelerator, SysMods},
       event::{Event, StartCause, WindowEvent},
       event_loop::{ControlFlow, EventLoop},
+      global_shortcut::ShortcutManager,
       keyboard::KeyCode,
       menu::{ContextMenu, MenuItemAttributes, MenuType},
-      platform::global_shortcut::ShortcutManager,
       system_tray::SystemTrayBuilder,
       window::{WindowBuilder, WindowId},
     },
@@ -131,7 +129,7 @@ fn main() -> wry::Result<()> {
 
       let webview = WebViewBuilder::new(window)
         .unwrap()
-        .with_custom_protocol("wry.dev".into(), move |_, _| {
+        .with_custom_protocol("wry.dev".into(), move |_uri| {
           Ok((index_html.as_bytes().into(), "text/html".into()))
         })
         .with_url("wry.dev://")
@@ -201,11 +199,6 @@ fn main() -> wry::Result<()> {
         }
         // click on `quit` item
         if menu_id == quit_item.clone().id() {
-          // on windows, we make sure to remove the icon from the tray
-          // it require the `SystemTrayExtWindows`
-          #[cfg(target_os = "windows")]
-          system_tray.remove();
-
           // tell our app to close at the end of the loop.
           *control_flow = ControlFlow::Exit;
         }
