@@ -8,9 +8,21 @@ mod web_context;
 
 pub use web_context::WebContext;
 
-#[cfg(target_os = "linux")]
+#[cfg(any(
+  target_os = "linux",
+  target_os = "dragonfly",
+  target_os = "freebsd",
+  target_os = "netbsd",
+  target_os = "openbsd"
+))]
 mod webkitgtk;
-#[cfg(target_os = "linux")]
+#[cfg(any(
+  target_os = "linux",
+  target_os = "dragonfly",
+  target_os = "freebsd",
+  target_os = "netbsd",
+  target_os = "openbsd"
+))]
 use webkitgtk::*;
 #[cfg(any(target_os = "macos", target_os = "ios"))]
 mod wkwebview;
@@ -31,9 +43,6 @@ use url::Url;
 #[cfg(target_os = "windows")]
 use crate::application::platform::windows::WindowExtWindows;
 use crate::application::window::Window;
-#[cfg(target_os = "windows")]
-#[cfg(feature = "winrt")]
-use windows_webview2::Windows::Win32::WindowsAndMessaging::HWND;
 
 pub struct WebViewAttributes {
   /// Whether the WebView window should be visible.
@@ -49,12 +58,9 @@ pub struct WebViewAttributes {
   /// Register custom file loading protocols with pairs of scheme uri string and a handling
   /// closure.
   ///
-  /// The closure takes the `Window` and a url string slice as parameters, and returns a tuple of a
-  /// vector of bytes which is the content and a mimetype string of the content.
-  pub custom_protocols: Vec<(
-    String,
-    Box<dyn Fn(&Window, &str) -> Result<(Vec<u8>, String)>>,
-  )>,
+  /// The closure takes a url string slice, and returns a two item tuple of a vector of
+  /// bytes which is the content and a mimetype string of the content.
+  pub custom_protocols: Vec<(String, Box<dyn Fn(&str) -> Result<(Vec<u8>, String)>>)>,
   /// Set the RPC handler to Communicate between the host Rust code and Javascript on webview.
   ///
   /// The communication is done via [JSON-RPC](https://www.jsonrpc.org). Users can use this to register an incoming
@@ -100,7 +106,7 @@ impl Default for WebViewAttributes {
 /// [`WebViewBuilder`] privides ability to setup initialization before web engine starts.
 pub struct WebViewBuilder<'a> {
   pub webview: WebViewAttributes,
-  web_context: Option<&'a WebContext>,
+  web_context: Option<&'a mut WebContext>,
   window: Window,
 }
 
@@ -140,12 +146,12 @@ impl<'a> WebViewBuilder<'a> {
   /// Register custom file loading protocols with pairs of scheme uri string and a handling
   /// closure.
   ///
-  /// The closure takes the `Window` and a url string slice as parameters, and returns a tuple of a
+  /// The closure takes a url string slice, and returns a two item tuple of a
   /// vector of bytes which is the content and a mimetype string of the content.
   #[cfg(feature = "protocol")]
   pub fn with_custom_protocol<F>(mut self, name: String, handler: F) -> Self
   where
-    F: Fn(&Window, &str) -> Result<(Vec<u8>, String)> + 'static,
+    F: Fn(&str) -> Result<(Vec<u8>, String)> + 'static,
   {
     self
       .webview
@@ -196,7 +202,7 @@ impl<'a> WebViewBuilder<'a> {
   }
 
   /// Set the web context that can share with multiple [`WebView`]s.
-  pub fn with_web_context(mut self, web_context: &'a WebContext) -> Self {
+  pub fn with_web_context(mut self, web_context: &'a mut WebContext) -> Self {
     self.web_context = Some(web_context);
     self
   }
@@ -284,7 +290,13 @@ pub struct WebView {
 // blocks and raw pointer properly.
 impl Drop for WebView {
   fn drop(&mut self) {
-    #[cfg(target_os = "linux")]
+    #[cfg(any(
+      target_os = "linux",
+      target_os = "dragonfly",
+      target_os = "freebsd",
+      target_os = "netbsd",
+      target_os = "openbsd"
+    ))]
     unsafe {
       use crate::application::platform::unix::WindowExtUnix;
       use gtk::prelude::WidgetExtManual;
@@ -333,14 +345,10 @@ impl WebView {
     Ok(())
   }
 
-  /// Resize the WebView manually. This is required on Windows because its WebView API doesn't
+  /// Resize the WebView manually. This is only required on Windows because its WebView API doesn't
   /// provide a way to resize automatically.
   pub fn resize(&self) -> Result<()> {
     #[cfg(target_os = "windows")]
-    #[cfg(feature = "winrt")]
-    self.webview.resize(HWND(self.window.hwnd() as _))?;
-    #[cfg(target_os = "windows")]
-    #[cfg(feature = "win32")]
     self.webview.resize(self.window.hwnd())?;
     Ok(())
   }
