@@ -46,23 +46,22 @@ pub struct InnerWebView {
   #[cfg(target_os = "macos")]
   ns_window: id,
   manager: id,
+  // Note that if following functions signatures are changed in the future,
+  // all fucntions pointer declarations in objc callbacks below all need to get updated.
   rpc_handler_ptr: *mut (
     Box<dyn Fn(&Window, RpcRequest) -> Option<RpcResponse>>,
     Rc<Window>,
   ),
   #[cfg(target_os = "macos")]
   file_drop_ptr: *mut (Box<dyn Fn(&Window, FileDropEvent) -> bool>, Rc<Window>),
-  protocol_ptrs: Vec<*mut (
-    Box<dyn Fn(&Window, &str) -> Result<(Vec<u8>, String)>>,
-    Rc<Window>,
-  )>,
+  protocol_ptrs: Vec<*mut Box<dyn Fn(&str) -> Result<(Vec<u8>, String)>>>,
 }
 
 impl InnerWebView {
   pub fn new(
     window: Rc<Window>,
     attributes: WebViewAttributes,
-    _web_context: Option<&WebContext>,
+    _web_context: Option<&mut WebContext>,
   ) -> Result<Self> {
     // Function for rpc handler
     extern "C" fn did_receive(this: &Object, _: Sel, _: id, msg: id) {
@@ -98,11 +97,8 @@ impl InnerWebView {
     extern "C" fn start_task(this: &Object, _: Sel, _webview: id, task: id) {
       unsafe {
         let function = this.get_ivar::<*mut c_void>("function");
-        let function = &mut *(*function
-          as *mut (
-            Box<dyn for<'r, 's> Fn(&'r Window, &'s str) -> Result<(Vec<u8>, String)>>,
-            Rc<Window>,
-          ));
+        let function =
+          &mut *(*function as *mut Box<dyn for<'s> Fn(&'s str) -> Result<(Vec<u8>, String)>>);
 
         // Get url request
         let request: id = msg_send![task, request];
@@ -114,7 +110,7 @@ impl InnerWebView {
         let uri = nsstring.to_str();
 
         // Send response
-        if let Ok((content, mime)) = function.0(&function.1, uri) {
+        if let Ok((content, mime)) = function(uri) {
           let dictionary: id = msg_send![class!(NSMutableDictionary), alloc];
           let headers: id = msg_send![dictionary, initWithCapacity:1];
           let () = msg_send![headers, setObject:NSString::new(&mime) forKey: NSString::new("content-type")];
@@ -163,8 +159,7 @@ impl InnerWebView {
           None => Class::get(&scheme_name).expect("Failed to get the class definition"),
         };
         let handler: id = msg_send![cls, new];
-        let w = window.clone();
-        let function = Box::into_raw(Box::new((function, w)));
+        let function = Box::into_raw(Box::new(function));
         protocol_ptrs.push(function);
 
         (*handler).set_ivar("function", function as *mut _ as *mut c_void);
